@@ -1,4 +1,20 @@
-#' Area-Proportional Euler Diagrams
+# eulerr: Area-Proportional Euler and Venn Diagrams with Circles or Ellipses
+# Copyright (C) 2018 Johan Larsson <johanlarsson@outlook.com>
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+#' Area-proportional Euler diagrams
 #'
 #' Fit Euler diagrams (a generalization of Venn diagrams) using numerical
 #' optimization to find exact or approximate solutions to a specification of set
@@ -21,8 +37,8 @@
 #' explored. The stress statistic can also be used as a goodness of fit
 #' measure.
 #'
-#' `euler()` also returns `diagError` and `regionError` from
-#' *eulerAPE*. `regionError` is computed as
+#' [euler()] also returns `diagError` and `regionError` from
+#' \pkg{eulerAPE}. `regionError` is computed as
 #'
 #' \deqn{
 #'     \left| \frac{y_i}{\sum y_i} - \frac{\hat{y}_i}{\sum \hat{y}_i}\right|.
@@ -32,38 +48,36 @@
 #'
 #' `diagError` is simply the maximum of regionError.
 #'
-#' @param combinations Set relationships as a named numeric vector, matrix, or
-#'   data.frame. (See the methods (by class) section for details.)
-#' @param by A factor or character matrix to be used in [base::by()] to
-#'   split the data.frame or matrix of set combinations.
-#' @param input The type of input: disjoint class combinations
-#'   (`disjoint`) or unions (`union`).
-#' @param shape The geometric shape used in the diagram: `circle` or `ellipse`.
-#' @param control A list of control parameters.
-#'   * `extraopt`: Should the more thorough optimizer (currently
+#' @param combinations set relationships as a named numeric vector, matrix, or
+#'   data.frame (see **methods (by class)**)
+#' @param by a factor or character matrix to be used in [base::by()] to
+#'   split the data.frame or matrix of set combinations
+#' @param input type of input: disjoint identities
+#'   (`'disjoint'`) or unions (`'union'`).
+#' @param shape geometric shape used in the diagram
+#' @param control a list of control parameters.
+#'   * `extraopt`: should the more thorough optimizer (currently
 #'   [RcppDE::DEoptim()]) kick in (provided `extraopt_threshold` is exceeded)? The
 #'   default is `TRUE` for ellipses and three sets and `FALSE` otherwise.
-#'   * `extraopt_threshold`: The threshold, in terms of `diagError`, for when
+#'   * `extraopt_threshold`: threshold, in terms of `diagError`, for when
 #'   the extra optimizer kicks in. This will almost always slow down the
 #'   process considerably. A value of 0 means
 #'   that the extra optimizer will kick in if there is *any* error. A value of
 #'   1 means that it will never kick in. The default is `0.001`.
-#'   * `extraopt_control`: A list of control parameters to pass to the
+#'   * `extraopt_control`: a list of control parameters to pass to the
 #'   extra optimizer, such as `itermax`. See [RcppDE::DEoptim.control()].
-#' @param ... Arguments passed down to other methods.
+#' @param ... arguments passed down to other methods
 #'
 #' @return A list object of class `'euler'` with the following parameters.
-#'   \item{coefficients}{A matrix of `h` and `k` (x and y-coordinates for the
-#'     centers of the
-#'     shapes) and, for circles, `r` for radii or, for ellipses, semiaxes `a`
-#'     and `b` and rotation angle `phi`.}
-#'   \item{original.values}{Set relationships provided by the user.}
-#'   \item{fitted.values}{Set relationships in the solution.}
-#'   \item{residuals}{Residuals.}
-#'   \item{diagError}{The largest absolute residual in percentage points
-#'     between the original and fitted areas.}
-#'   \item{stress}{The stress of the solution, computed as the sum of squared
-#'     residuals over the total sum of squares.}
+#'   \item{coefficients}{a matrix of `h` and `k` (x and y-coordinates for the
+#'     centers of the shapes), semiaxes `a` and `b`, and rotation angle `phi`}
+#'   \item{original.values}{set relationships in the input}
+#'   \item{fitted.values}{set relationships in the solution}
+#'   \item{residuals}{residuals}
+#'   \item{regionError}{the difference in percentage points between each
+#'     disjoint subset in the input and the respective area in the output}
+#'   \item{diagError}{the largest `regionError`}
+#'   \item{stress}{normalized residual sums of squares}
 #'
 #' @seealso [plot.euler()], [print.euler()]
 #'
@@ -84,26 +98,6 @@
 #' # Plot it
 #' plot(fit2)
 #'
-#' # A euler diagram from a list of sample spaces (the list method)
-#' euler(list(A = c("a", "ab", "ac", "abc"),
-#'            B = c("b", "ab", "bc", "abc"),
-#'            C = c("c", "ac", "bc", "abc")))
-#'
-#' # Using the matrix method
-#' mat <- cbind(A = sample(c(TRUE, TRUE, FALSE), size = 50, replace = TRUE),
-#'              B = sample(c(TRUE, FALSE), size = 50, replace = TRUE))
-#' euler(mat)
-#'
-#' # Using grouping via the 'by' argument
-#' dat <- data.frame(
-#'   A = sample(c(TRUE, FALSE), size = 100, replace = TRUE),
-#'   B = sample(c(TRUE, TRUE, FALSE), size = 100, replace = TRUE),
-#'   gender = sample(c("Men", "Women"), size = 100, replace = TRUE),
-#'   nation = sample(c("Sweden", "Denmark"), size = 100, replace = TRUE)
-#' )
-#'
-#' euler(dat[, 1:2], by = dat[, 3:4])
-#'
 #' # A set with no perfect solution
 #' euler(c("a" = 3491, "b" = 3409, "c" = 3503,
 #'         "a&b" = 120, "a&c" = 114, "b&c" = 132,
@@ -122,7 +116,7 @@
 #' @export
 euler <- function(combinations, ...) UseMethod("euler")
 
-#' @describeIn euler A named numeric vector, with
+#' @describeIn euler a named numeric vector, with
 #'   combinations separated by an ampersand, for instance `A&B = 10`.
 #'   Missing combinations are treated as being 0.
 #'
@@ -266,6 +260,8 @@ euler.default <- function(
       ),
       byrow = TRUE
     ))
+    if (circle)
+      tpar <- cbind(tpar, tpar[, 3L], 0)
 
     # Normalize layout
     nlm_fit <- as.vector(intersect_ellipses(nlm_solution, circle))
@@ -341,15 +337,14 @@ euler.default <- function(
     diagError <- diagError(regionError = regionError)
     stress <- stress(orig, fit)
 
-    fpar <- as.data.frame(matrix(
-      data = final_par,
-      ncol = if (circle) 3L else 5L,
-      dimnames = list(
-        setnames,
-        if (circle) c("h", "k", "r") else c("h", "k", "a", "b", "phi")
-      ),
-      byrow = TRUE
-    ))
+    fpar <- matrix(data = final_par,
+                   ncol = if (circle) 3L else 5L,
+                   byrow = TRUE)
+
+    if (circle)
+      fpar <- cbind(fpar, fpar[, 3L], 0)
+
+    dimnames(fpar) <- list(setnames, c("h", "k", "a", "b", "phi"))
 
     # Normalize semiaxes and rotation
     fpar <- normalize_pars(fpar)
@@ -360,27 +355,17 @@ euler.default <- function(
     # Center the solution on the coordinate plane
     fpar <- center_layout(fpar)
   } else {
-    circle <- match.arg(shape) == "circle"
     # One set
-    fpar <- as.data.frame(matrix(
-      data = if (circle)
-        c(0, 0, sqrt(areas/pi))
-      else
-        c(0, 0, sqrt(areas/pi), sqrt(areas/pi), 0),
-      ncol = if (circle) 3L else 5L,
-      dimnames = list(
-        setnames,
-        if (circle) c("h", "k", "r") else c("h", "k", "a", "b", "phi")
-      ),
-      byrow = TRUE
-    ))
+    fpar <- matrix(data = c(0, 0, sqrt(areas/pi), sqrt(areas/pi), 0),
+                   ncol = 5L,
+                   dimnames = list(setnames, c("h", "k", "a", "b", "phi")))
     regionError <- diagError <- stress <- 0
     orig <- fit <- areas
     names(orig) <- names(fit) <- setnames
   }
 
   # Return eulerr structure
-  structure(list(coefficients = fpar,
+  structure(list(ellipses = as.data.frame(fpar),
                  original.values = orig,
                  fitted.values = fit,
                  residuals = orig - fit,
@@ -390,59 +375,118 @@ euler.default <- function(
             class = c("euler", "list"))
 }
 
-#' @describeIn euler A data.frame of logicals, two-level factors (see examples).
-#' @param weights A numeric vector of weights of the same length as `by` and
+#' @describeIn euler a data.frame of logicals, two-level factors (see examples).
+#' @param weights a numeric vector of weights of the same length as
 #'   the number of rows in `combinations`.
 #' @export
+#' @examples
+#' # Using grouping via the 'by' argument through the data.frame method
+#' dat <- data.frame(
+#'   A = sample(c(TRUE, FALSE), size = 100, replace = TRUE),
+#'   B = sample(c(TRUE, TRUE, FALSE), size = 100, replace = TRUE),
+#'   gender = sample(c("Men", "Women"), size = 100, replace = TRUE),
+#'   nation = sample(c("Sweden", "Denmark"), size = 100, replace = TRUE)
+#' )
+#'
+#' euler(dat, by = list(gender, nation))
+#'
+#'
+#' dat2 <- data.frame(A = c(TRUE, FALSE, TRUE, TRUE),
+#'                    B = c(FALSE, TRUE, TRUE, FALSE))
+#' euler(dat2, weights = c(3, 2, 1, 1))
+#'
 euler.data.frame <- function(combinations, weights = NULL, by = NULL, ...) {
   stopifnot(!any(grepl("&", colnames(combinations), fixed = TRUE)))
 
-  if (is.null(weights))
-    weights <- rep.int(1L, nrow(combinations))
+  facs <- eval(substitute(by), combinations)
 
-  if (!is.null(by)) {
-    stopifnot(all(vapply(by,
-                         function(x) (is.factor(x) || is.character(x)),
-                         FUN.VALUE = logical(1))))
-    if (NCOL(by) > 2L)
-      stop("No more than two conditioning variables are allowed.")
-  }
+  if (!is.null(facs)) {
+    if (is.list(facs)) {
+      if (!is.null(names(facs)))
+        nms <- names(facs)
+      else {
+        nms <- sapply(substitute(by)[-1L], deparse)
+      }
+    } else
+      nms <- deparse(substitute(by))
 
-  out <- matrix(NA, nrow = nrow(combinations), ncol = ncol(combinations))
-  colnames(out) <- colnames(combinations)
+    if (is.list(facs))
+      stopifnot(length(facs) < 3)
+    else
+      facs <- list(facs)
 
-  for (i in seq_along(combinations)) {
-    y <- combinations[, i]
-    if (is.factor(y) || is.character(y)) {
-      facs <- unique(as.character(y))
-      if (length(facs) > 2L)
-        stop("No more than 2 levels allowed.")
-      out[, i] <- y == facs[1L]
-      colnames(out)[i] <- facs[1L]
-    } else if (is.numeric(y)) {
-      out[, i] <- as.logical(y)
-    } else if (is.logical(y)) {
-      out[, i] <- y
-    } else {
-      stop("Unsupported type of variables.")
+    dd <- as.data.frame(facs, col.names = nms)
+    groups <- unique(dd)
+    rownames(groups) <- NULL
+
+    out <- g <- vector("list", NROW(groups))
+    int_or_log <- vapply(combinations,
+                         function(x) is.numeric(x) || is.logical(x),
+                         FUN.VALUE = logical(1L))
+    for (i in seq_len(NROW(groups))) {
+      ind <- apply(dd, 1, function(x) all(x == groups[i, ]))
+      out[[i]] <- euler(combinations[ind, int_or_log])
+      names(out)[[i]] <- paste(unlist(groups[i, , drop = TRUE]),
+                               collapse = ".")
     }
-  }
-  combinations <- as.data.frame(out)
-  combinations$weights <- weights
 
-  if (is.null(by)) {
-    out <- tally_combinations(combinations)
+    class(out) <- c("euler", "list")
+    attr(out, "groups") <- groups
   } else {
-    out <- by(combinations, by, tally_combinations, simplify = FALSE)
-    class(out) <- c("by", "euler", "list")
-  }
+    if (is.null(weights)) {
+      combinations <- combinations[,
+                                   vapply(combinations,
+                                          function(y)
+                                            is.integer(y) || is.logical(y),
+                                          FUN.VALUE = logical(1L))]
+    } else {
+      combinations <- combinations[,
+                                   vapply(combinations,
+                                          function(y)
+                                            is.integer(y) ||
+                                            is.logical(y) ||
+                                            is.factor(y) ||
+                                            is.character(y),
+                                          FUN.VALUE = logical(1L))]
+    }
 
+    if (is.null(weights))
+      weights <- rep.int(1L, NROW(combinations))
+
+    out <- matrix(NA, nrow = NROW(combinations), ncol = NCOL(combinations))
+    colnames(out) <- colnames(combinations)
+
+    for (i in seq_along(combinations)) {
+      y <- combinations[, i]
+      if (is.factor(y) || is.character(y)) {
+        facs <- unique(as.character(y))
+        if (length(facs) > 2L)
+          stop("no more than 2 levels allowed")
+        out[, i] <- y == facs[1L]
+        colnames(out)[i] <- facs[1L]
+      } else if (is.numeric(y)) {
+        out[, i] <- as.logical(y)
+      } else if (is.logical(y)) {
+        out[, i] <- y
+      } else {
+        stop("unsupported type of variables")
+      }
+    }
+    combinations <- as.data.frame(out)
+    out <- tally_combinations(combinations, weights)
+    out <- euler(out, ...)
+  }
   out
 }
 
-#' @describeIn euler A matrix that can be converted to a data.frame of logicals
+#' @describeIn euler a matrix that can be converted to a data.frame of logicals
 #'   (as in the description above) via [base::as.data.frame.matrix()].
 #' @export
+#' @examples
+#' # Using the matrix method
+#' mat <- cbind(A = sample(c(TRUE, TRUE, FALSE), size = 50, replace = TRUE),
+#'              B = sample(c(TRUE, FALSE), size = 50, replace = TRUE))
+#' euler(mat)
 euler.matrix <- function(combinations, ...) {
   euler(as.data.frame(combinations), ...)
 }
@@ -454,14 +498,19 @@ euler.matrix <- function(combinations, ...) {
 #' plot(euler(as.table(apply(Titanic, 2:4, sum))))
 euler.table <- function(combinations, ...) {
   if (max(dim(combinations)) > 2L)
-    stop("No table dimension may exceed 2.")
+    stop("no table dimension may exceed 2")
   x <- as.data.frame(combinations)
   euler(x[, !(names(x) == "Freq")], weights = x$Freq, ...)
 }
 
-#' @describeIn euler A list of vectors, each vector giving the contents of
+#' @describeIn euler a list of vectors, each vector giving the contents of
 #'   that set (with no duplicates). Vectors in the list do not need to be named.
 #' @export
+#' @examples
+#' # A euler diagram from a list of sample spaces (the list method)
+#' euler(list(A = c("a", "ab", "ac", "abc"),
+#'            B = c("b", "ab", "bc", "abc"),
+#'            C = c("c", "ac", "bc", "abc")))
 euler.list <- function(combinations, ...) {
   stopifnot(!is.null(attr(combinations, "names")),
             !any(names(combinations) == ""),
